g.type-TK>rect {
    fill: #00ffd0;
}

text {
    font-weight: 300;
    font-family: "Helvetica Neue", Helvetica, Arial, sans-serf;
    font-size: 14px;
}

.node rect {
    stroke: #999;
    fill: #fff;
    stroke-width: 1.5px;
}

.edgePath path {
    stroke: #333;
    stroke-width: 1.5px;
}

h1 {
    color: #111;
    font-family: 'Helvetica Neue', sans-serif;
    font-size: 75px;
    font-weight: bold;
    letter-spacing: -1px;
    line-height: 1;
    text-align: center;
}

h2 {
    color: #111;
    font-family: 'Helvetica Neue', sans-serif;
    font-size: 20px;
    margin: 0 0 72px;
    text-align: center;
}